diff -u -r vlc-1.1.11/src/network/tcp.c vlc-1.1.11-smartconnect/src/network/tcp.c
--- vlc-1.1.11/src/network/tcp.c	2010-05-06 04:38:21.000000000 -0400
+++ vlc-1.1.11-smartconnect/src/network/tcp.c	2011-08-27 13:15:00.814848949 -0400
@@ -66,6 +66,196 @@
                        int i_protocol );
 
 #undef net_Connect
+
+#define SMART_CONNECT 1
+#define SMART_CONNECT_USE_CACHE 1
+
+#ifdef SMART_CONNECT
+/******************************************************************************
+*
+* (c) Copyright 2006-2011 CERTARETE, All Rights Reserved.
+*
+* DESCRIPTION
+* Concurrently connect to multiple TCP servers
+* and return the socket for the server that was the first
+* to respond to the connection request, which means this
+* is either the closest server to the client or the least
+* loaded server of a nearby group.
+*
+* NOTES
+* This implementation should support both IPv4 and IPv6,
+* although only IPv4 was tested.
+*
+* AUTHOR
+* Leonid Rosenboim <Leonid@Certarete.com>
+* Implementation of Smart connect cache and porting to VLC:
+* Kiran Kotla <kotlakiran@gmail.com>
+*
+*/
+#include <sys/fcntl.h>
+#define	SMART_CONN_MAX_CONN	16	/* Maximum concurrent connections */
+
+#if	SMART_CONN_MAX_CONN > FD_SETSIZE
+#error	SMART_CONN_MAX_CONN is too large
+#endif
+
+#ifndef	FD_COPY
+#define	FD_COPY(s,d) (memcpy((d),(s),sizeof(d)))
+#endif
+#define MAX_RESOLVE_CACHE_ENTRIES 16
+#define MAX_HOSTNAME_CHAR       256
+#define CACHE_ENTRY_TIMEOUT     120    /* 2 minute timeout for the cache entry */
+
+struct _smart_conn_s {
+	int 		sock_fd ;
+	struct timeval 	t_rx ;
+	unsigned	flags ;
+#ifdef SMART_CONNECT_USE_CACHE
+	struct addrinfo *res;
+#endif
+} ;
+
+static struct _smart_conn_s _smart_conn_tab[ SMART_CONN_MAX_CONN ] ;
+
+#ifdef SMART_CONNECT_USE_CACHE
+struct resolve_cache_t {
+    char hostname[MAX_HOSTNAME_CHAR];
+    time_t last_accessed;
+    time_t last_updated;
+    int ai_family;
+    int ai_socktype;
+    int ai_protocol;
+    size_t ai_addrlen;
+    struct sockaddr_storage ai_addr;
+};
+
+static struct resolve_cache_t resolve_cache[MAX_RESOLVE_CACHE_ENTRIES];
+int num_resolve_cache_entries=0;
+
+static struct resolve_cache_t* find_in_resolve_cache(char *hostname)
+{
+	int i;
+	fprintf(stdout, "Retrieving from cache\n");
+	for (i=0; i<num_resolve_cache_entries; i++) {
+		if (strcmp(resolve_cache[i].hostname, hostname) == 0) {
+			/* found a match */
+			time_t cur_time = time(NULL);
+			if ( (cur_time - resolve_cache[i].last_updated) < CACHE_ENTRY_TIMEOUT) {
+				/* update the access time */
+				fprintf(stdout, "Match found in the cache: %d\n", num_resolve_cache_entries);
+				resolve_cache[i].last_accessed = cur_time;
+				return (resolve_cache+i);
+			}
+			else
+				return NULL;   /* update_in_resolve_cache will take care of removing this entry */
+		}
+	}
+	fprintf(stdout, "Did not find in the cache: %d\n", num_resolve_cache_entries);
+	return NULL;
+}
+
+static void copy_to_cache_entry(struct resolve_cache_t *cache_entry, struct addrinfo *res)
+{
+	cache_entry->ai_family = res->ai_family;
+	cache_entry->ai_socktype = res->ai_socktype;
+	cache_entry->ai_protocol = res->ai_protocol;
+	cache_entry->ai_addrlen = res->ai_addrlen;
+	bzero(&(cache_entry->ai_addr), sizeof(cache_entry->ai_addr));
+	memcpy(&(cache_entry->ai_addr), res->ai_addr, res->ai_addrlen);
+}
+
+static void update_in_resolve_cache(char *hostname, struct addrinfo *res)
+{
+	int i;
+	time_t cur_time = time(NULL);
+	int oldest_accessed_index = 0;
+	fprintf(stdout, "Adding to cache\n");
+
+	for (i=0;i<num_resolve_cache_entries;i++) {
+		if (strcmp(resolve_cache[i].hostname, hostname) == 0) {
+			/* found a match, update the last accessed time */
+			copy_to_cache_entry(resolve_cache+i, res);
+			resolve_cache[i].last_updated = resolve_cache[i].last_accessed = cur_time;
+			return;
+		}
+	}
+		
+	/* now look if there is an entry in the cache that can be replaced */
+	cur_time = time(NULL);
+	for (i=0;i<num_resolve_cache_entries;i++) {
+		if ( (cur_time - resolve_cache[i].last_updated) >= CACHE_ENTRY_TIMEOUT) {
+			/* time out for this entry, replace it!! */
+			copy_to_cache_entry(resolve_cache+i, res);
+			strcpy(resolve_cache[i].hostname, hostname);
+			resolve_cache[i].last_updated = resolve_cache[i].last_accessed = cur_time;
+			return;
+		}
+	}
+
+	/* now look if there is any space in the cache */
+	if (num_resolve_cache_entries < MAX_RESOLVE_CACHE_ENTRIES) {
+		copy_to_cache_entry(resolve_cache+num_resolve_cache_entries, res);
+		strcpy(resolve_cache[num_resolve_cache_entries].hostname, hostname);
+		resolve_cache[num_resolve_cache_entries].last_updated = resolve_cache[num_resolve_cache_entries].last_accessed = cur_time;
+		num_resolve_cache_entries++;
+		return;
+	}
+	
+	/* there is no space, replace the oldest accessed entry */		
+	for (i=1;i<num_resolve_cache_entries;i++) {
+		if (resolve_cache[i].last_accessed < resolve_cache[oldest_accessed_index].last_accessed)
+			oldest_accessed_index = i;
+	}
+
+	/* do the replacing */
+	copy_to_cache_entry(resolve_cache+oldest_accessed_index, res);
+	strcpy(resolve_cache[oldest_accessed_index].hostname, hostname);
+	resolve_cache[oldest_accessed_index].last_updated = resolve_cache[oldest_accessed_index].last_accessed = cur_time;
+}
+
+static int poll_and_check(vlc_object_t *p_this, int fd, int evfd)
+{
+    int val;
+    int timeout = var_InheritInteger (p_this, "ipv4-timeout");
+    struct pollfd ufd[2] = {
+        { .fd = fd,   .events = POLLOUT },
+        { .fd = evfd, .events = POLLIN },
+    };
+
+    do
+        /* NOTE: timeout screwed up if we catch a signal (EINTR) */
+        val = poll (ufd, sizeof (ufd) / sizeof (ufd[0]), timeout);
+    while ((val == -1) && (net_errno == EINTR));
+
+    switch (val)
+    {
+        case -1: /* error */
+            msg_Err (p_this, "connection polling error: %m");
+            return 0;
+
+        case 0: /* timeout */
+            msg_Warn (p_this, "connection timed out");
+            return 0;
+
+        default: /* something happended */
+            if (ufd[1].revents)
+                return 0; /* LibVLC object killed */
+    }
+
+    /* There is NO WAY around checking SO_ERROR.
+     * Don't ifdef it out!!! */
+    if (getsockopt (fd, SOL_SOCKET, SO_ERROR, &val,
+                       &(socklen_t){ sizeof (val) }) || val)
+    {
+        errno = val;
+        msg_Err (p_this, "connection failed: %m");
+        return 0;
+    }
+    return 1;
+}
+#endif
+
+#endif
 /*****************************************************************************
  * net_Connect:
  *****************************************************************************
@@ -75,7 +265,7 @@
 int net_Connect( vlc_object_t *p_this, const char *psz_host, int i_port,
                  int type, int proto )
 {
-    struct addrinfo hints, *res, *ptr;
+    struct addrinfo hints, *res=NULL, *ptr;
     const char      *psz_realhost;
     char            *psz_socks;
     int             i_realport, i_val, i_handle = -1;
@@ -137,6 +327,47 @@
                  i_realport );
     }
 
+#ifdef SMART_CONNECT
+    const char *cause = NULL;
+    static char cause_str[ 32 ];
+    fd_set fds ;
+    unsigned index, conn_count ;
+    int fd_max;
+    int error;
+    int opt;
+#ifdef SMART_CONNECT_USE_CACHE
+    int match_index = -1;
+    struct resolve_cache_t *cache_entry = find_in_resolve_cache(psz_realhost);
+
+    if (cache_entry) {
+        int fd = net_Socket( p_this, cache_entry->ai_family,
+                             cache_entry->ai_socktype, cache_entry->ai_protocol );
+	fprintf(stdout, "Found a match in the cache\n");
+	if (fd > 0) {
+		fprintf(stdout, "matched socket succeeds\n");
+        	if(connect( fd, (struct sockaddr *)&(cache_entry->ai_addr), cache_entry->ai_addrlen ) == 0 || (net_errno == EINPROGRESS && poll_and_check(p_this, fd, evfd))) {
+			fprintf(stdout, "matched connect succeeds\n");
+        		i_handle = fd; /* success! */
+			goto finish;
+		}
+		fprintf(stdout, "Error: %s Address length: %d\n", strerror(errno), cache_entry->ai_addrlen);
+	}
+	/* something is wrong, just proceed normally */
+    }
+#endif
+    /* Clear conenction table */
+    memset( _smart_conn_tab, 0, sizeof(_smart_conn_tab ) );
+    for( index = 0; index < SMART_CONN_MAX_CONN; index ++ )
+        _smart_conn_tab[ index ]. sock_fd = -1 ;
+
+    /* Initialize the addrinfo hints structure */
+    memset(&hints, 0, sizeof(hints));
+    hints.ai_family = PF_UNSPEC;
+    hints.ai_socktype = SOCK_STREAM;
+    int fd = -1;
+    index = 0 ;
+#endif
+
     i_val = vlc_getaddrinfo( p_this, psz_realhost, i_realport, &hints, &res );
     free( psz_socks );
 
@@ -150,6 +381,164 @@
     int timeout = var_InheritInteger (p_this, "ipv4-timeout");
     if (timeout < 0)
         timeout = -1;
+#ifdef SMART_CONNECT
+    for( ptr = res; ptr != NULL; ptr = ptr->ai_next )
+    {
+        fd = net_Socket( p_this, ptr->ai_family,
+                             ptr->ai_socktype, ptr->ai_protocol );
+        if( fd == -1 )
+        {
+            msg_Dbg( p_this, "socket error: %m" );
+            continue;
+        }
+
+        /* Mark this socket non-blocking */
+        opt = fcntl( fd, F_GETFL, 0 );
+        if( opt < 0 || fcntl( fd, F_SETFL, opt | O_NONBLOCK )  < 0 )
+        {
+            msg_Dbg( p_this, "Error setting an error non-blocking: %s", strerror(errno) );
+            close (fd);
+            fd = -1;
+            continue ;
+        }
+
+        /* Initiate connection attempt */
+        error = connect(fd, ptr->ai_addr, ptr->ai_addrlen) ;
+
+        if( error < 0 && errno != EINPROGRESS )
+        {
+            cause = "connect";
+            close(fd);
+            fd = -1;
+            continue;
+        }
+
+        /* Save pending connection socket for later polling */
+#ifdef SMART_CONNECT_USE_CACHE
+        _smart_conn_tab[ index ].res = ptr ;   /* save the res ptr to use later to update the cache */
+#endif
+        _smart_conn_tab[ index ++ ].sock_fd = fd ;
+
+        /* Limit the number of outstanding conections */
+        if( index >= SMART_CONN_MAX_CONN )
+            break;
+
+        /* Should a server be faster than the client */
+        if( error == 0 )
+            break ;
+    } /* for res */
+
+    /* If there was no success in initiating a conenction, report an error */
+    if (fd < 0 && index == 0 )
+    {
+        goto smart_connect_error;
+    }
+
+    /* Prepare the select fd mask and counters */
+    FD_ZERO( & fds );
+    fd_max = 0 ;
+
+    for( conn_count = index = 0; index < SMART_CONN_MAX_CONN; index ++ )
+    {
+        if( (fd = _smart_conn_tab[index].sock_fd) < 0 )
+            continue ;
+        if( fd_max < fd )
+            fd_max = fd ;
+        FD_SET( fd, & fds );
+        conn_count ++ ;
+    }
+
+    /* Wait as long as at least one connection is stipp pending */
+    while( conn_count > 0 )
+    {
+        fd_set rfds, wfds, efds ;
+
+        /* Reiniitialize fd mask in each iteration */
+        FD_COPY( & fds, & rfds );
+        FD_COPY( & fds, & wfds );
+        FD_COPY( & fds, & efds );
+
+        /* NOTE: no time-out is set here, relying instead on TCP connection
+         * attempt to time out and indicate an error
+         */
+
+        error = select( fd_max+1, & rfds, & wfds, & efds, NULL );
+
+        if( error == 0 )
+        {
+            cause = "Timeout";
+            goto smart_connect_error;
+        }
+
+        if( error < 0 && errno == EINTR )
+	    continue ;
+
+        if( error < 0 )
+        {
+            strncpy( cause_str, strerror( errno ), sizeof(cause_str)-1 );
+            goto smart_connect_error;
+        }
+
+        fd = -1 ;
+        /* Inspect the results from select */
+        for( index = 0; index < SMART_CONN_MAX_CONN; index ++ )
+        {
+            if( (fd = _smart_conn_tab[index].sock_fd) < 0 )
+	        continue ;
+            if( FD_ISSET( fd, & efds ) )
+            {
+	        FD_CLR( fd, & fds );
+	        _smart_conn_tab[ index ].sock_fd = -1 ;
+                conn_count -- ;
+	        fd = -1 ;
+            }
+            else if( FD_ISSET( fd, & rfds ) || FD_ISSET( fd, & wfds ) )
+            {
+	        /* Live connection found */
+	        goto smart_connect_success ;
+	    }
+
+        } /* for index */
+
+    } /* while conn_count */
+    cause = "connection failed" ;
+smart_connect_error:
+    /* Close all sockets */
+    for( index = 0; index < SMART_CONN_MAX_CONN; index ++ )
+    {
+        if( _smart_conn_tab[index].sock_fd >= 0 )
+            (void) close( _smart_conn_tab[index].sock_fd );
+        _smart_conn_tab[index].sock_fd = -1 ;
+    }
+    msg_Err (p_this, "connection failed: %s", cause);
+    i_handle = -1;
+    goto finish;
+smart_connect_success:
+    /* <fd> connected, close all other pending sockets */
+    for( index = 0; index < SMART_CONN_MAX_CONN; index ++ )
+    {
+        if( fd == _smart_conn_tab[index].sock_fd ) {
+#ifdef SMART_CONNECT_USE_CACHE
+            match_index = index;    /* save the index of the sock_fd that matches with fd */
+#endif
+            continue ;
+	}
+        if( _smart_conn_tab[index].sock_fd < 0 )
+            continue ;
+        (void) close( _smart_conn_tab[index].sock_fd );
+        _smart_conn_tab[index].sock_fd = -1 ;
+    }
+
+    /* Revert the resulting connection to blocking mode */
+    opt = fcntl( fd, F_GETFL, 0 );
+    (void) fcntl( fd, F_SETFL, opt & ~O_NONBLOCK );
+    i_handle = fd;
+#ifdef SMART_CONNECT_USE_CACHE
+    if (match_index > -1)
+        update_in_resolve_cache(psz_realhost, _smart_conn_tab[match_index].res);
+#endif
+    goto finish;
+#endif
 
     for( ptr = res; ptr != NULL; ptr = ptr->ai_next )
     {
@@ -216,6 +605,7 @@
         continue;
     }
 
+finish:
     vlc_freeaddrinfo( res );
 
     if( i_handle == -1 )
